@@ -1,6 +1,15 @@
 //! Types and functions to configure a Tor client.
 //!
 //! Some of these are re-exported from lower-level crates.
+//!
+//! # ⚠ Stability Warning ⚠
+//!
+//! The design of this structure, and of the configuration system for
+//! Arti, is likely to change significantly before the release of Arti
+//! 1.0.0. The layout of options within this structure is also likely
+//! to change. For more information see ticket [#285].
+//!
+//! [#285]: https://gitlab.torproject.org/tpo/core/arti/-/issues/285
 
 use derive_builder::Builder;
 use serde::Deserialize;
@@ -8,9 +17,7 @@ use std::collections::HashMap;
 use std::path::Path;
 use std::path::PathBuf;
 use std::time::Duration;
-use tor_config::CfgPath;
-
-pub use tor_config::ConfigBuildError;
+pub use tor_config::{CfgPath, ConfigBuildError, Reconfigure};
 
 /// Types for configuring how Tor circuits are built.
 pub mod circ {
@@ -209,7 +216,7 @@ impl StorageConfig {
     }
     /// Try to expand `cache_dir` to be a path buffer.
     pub(crate) fn expand_cache_dir(&self) -> Result<PathBuf, ConfigBuildError> {
-        self.state_dir
+        self.cache_dir
             .path()
             .map_err(|e| ConfigBuildError::Invalid {
                 field: "cache_dir".to_owned(),
@@ -226,6 +233,46 @@ impl From<StorageConfig> for StorageConfigBuilder {
     }
 }
 
+/// Configuration for system resources used by Tor.
+///
+/// You cannot change this section on a running Arti client.
+#[derive(Deserialize, Debug, Clone, Builder, Eq, PartialEq)]
+#[serde(deny_unknown_fields)]
+#[builder(build_fn(error = "ConfigBuildError"))]
+#[non_exhaustive]
+pub struct SystemConfig {
+    /// Maximum number of file descriptors we should launch with
+    #[builder(setter(into), default = "default_max_files()")]
+    #[serde(default = "default_max_files")]
+    pub max_files: u64,
+}
+
+/// Return the default maximum number of file descriptors to launch with.
+fn default_max_files() -> u64 {
+    16384
+}
+
+impl Default for SystemConfig {
+    fn default() -> Self {
+        Self::builder().build().expect("Default builder failed")
+    }
+}
+
+impl SystemConfig {
+    /// Return a new SystemConfigBuilder.
+    pub fn builder() -> SystemConfigBuilder {
+        SystemConfigBuilder::default()
+    }
+}
+
+impl From<SystemConfig> for SystemConfigBuilder {
+    fn from(cfg: SystemConfig) -> SystemConfigBuilder {
+        let mut builder = SystemConfigBuilder::default();
+        builder.max_files(cfg.max_files);
+        builder
+    }
+}
+
 /// A configuration used to bootstrap a [`TorClient`](crate::TorClient).
 ///
 /// In order to connect to the Tor network, Arti needs to know a few
@@ -244,8 +291,14 @@ impl From<StorageConfig> for StorageConfigBuilder {
 /// Finally, you can get fine-grained control over the members of a a
 /// TorClientConfig using [`TorClientConfigBuilder`].
 ///
-/// NOTE: These are NOT the final options or their final layout.
-/// Expect NO stability here.
+/// # ⚠ Stability Warning ⚠
+///
+/// The design of this structure, and of the configuration system for
+/// Arti, is likely to change significantly before the release of Arti
+/// 1.0.0. The layout of options within this structure is also likely
+/// to change. For more information see ticket [#285].
+///
+/// [#285]: https://gitlab.torproject.org/tpo/core/arti/-/issues/285
 #[derive(Clone, Debug, Eq, PartialEq)]
 pub struct TorClientConfig {
     /// Information about the Tor network we want to connect to.
@@ -275,6 +328,9 @@ pub struct TorClientConfig {
 
     /// Information about timing out client requests.
     pub(crate) stream_timeouts: StreamTimeoutConfig,
+
+    /// Information about system resources
+    pub system: SystemConfig,
 }
 
 impl Default for TorClientConfig {
@@ -338,6 +394,8 @@ pub struct TorClientConfigBuilder {
     address_filter: ClientAddrConfigBuilder,
     /// Inner builder for the `stream_timeouts` section.
     stream_timeouts: StreamTimeoutConfigBuilder,
+    /// Inner builder for the `system` section.
+    system: SystemConfigBuilder,
 }
 
 impl TorClientConfigBuilder {
@@ -373,6 +431,7 @@ impl TorClientConfigBuilder {
             .stream_timeouts
             .build()
             .map_err(|e| e.within("stream_timeouts"))?;
+        let system = self.system.build().map_err(|e| e.within("system"))?;
 
         Ok(TorClientConfig {
             tor_network,
@@ -384,6 +443,7 @@ impl TorClientConfigBuilder {
             circuit_timing,
             address_filter,
             stream_timeouts,
+            system,
         })
     }
 
@@ -487,6 +547,13 @@ impl TorClientConfigBuilder {
     pub fn address_filter(&mut self) -> &mut ClientAddrConfigBuilder {
         &mut self.address_filter
     }
+
+    /// Return a mutable reference to a [`SystemConfigBuilder`].
+    ///
+    /// This section is used to configure the system resources used by Arti.
+    pub fn system(&mut self) -> &mut SystemConfigBuilder {
+        &mut self.system
+    }
 }
 
 impl From<TorClientConfig> for TorClientConfigBuilder {
@@ -501,6 +568,7 @@ impl From<TorClientConfig> for TorClientConfigBuilder {
             circuit_timing,
             address_filter,
             stream_timeouts,
+            system,
         } = cfg;
 
         TorClientConfigBuilder {
@@ -513,6 +581,7 @@ impl From<TorClientConfig> for TorClientConfigBuilder {
             circuit_timing: circuit_timing.into(),
             address_filter: address_filter.into(),
             stream_timeouts: stream_timeouts.into(),
+            system: system.into(),
         }
     }
 }
